"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createQueue = exports.execRedisCommand = exports.getRedisClient = exports.getRedisInfo = exports.ping = exports.getConnectionQueues = void 0;
const ioredis_1 = require("ioredis");
const utils_1 = require("./utils");
const bullmq_1 = require("bullmq");
const Bull = require("bull");
const responders_1 = require("./responders");
const chalk = require("chalk");
const queueNameRegExp = new RegExp("(.*):(.*):id");
const maxCount = 50000;
const maxTime = 30000;
let redisClient;
const getQueueKeys = (client) => __awaiter(void 0, void 0, void 0, function* () {
    let keys = [], cursor = "0";
    const startTime = Date.now();
    do {
        const [nextCursor, scannedKeys] = yield client.scan(cursor, "MATCH", "*:*:id", "COUNT", maxCount);
        cursor = nextCursor;
        keys.push(...scannedKeys);
    } while (Date.now() - startTime < maxTime && cursor !== "0");
    return keys;
});
function getConnectionQueues(redisOpts, clusterNodes) {
    return __awaiter(this, void 0, void 0, function* () {
        const queues = yield execRedisCommand(redisOpts, (client) => __awaiter(this, void 0, void 0, function* () {
            const keys = yield getQueueKeys(client);
            const queues = yield Promise.all(keys
                .map(function (key) {
                var match = queueNameRegExp.exec(key);
                if (match) {
                    return {
                        prefix: match[1],
                        name: match[2],
                        type: "bull",
                    };
                }
            })
                .filter((queue) => queue !== undefined)
                .map(function (queue) {
                return __awaiter(this, void 0, void 0, function* () {
                    const type = yield (0, utils_1.getQueueType)(queue.name, queue.prefix, client);
                    queue.type = type;
                    return queue;
                });
            }));
            return queues;
        }), clusterNodes);
        return queues;
    });
}
exports.getConnectionQueues = getConnectionQueues;
function ping(redisOpts, clusterNodes) {
    return __awaiter(this, void 0, void 0, function* () {
        return execRedisCommand(redisOpts, (client) => client.ping(), clusterNodes);
    });
}
exports.ping = ping;
function getRedisInfo(redisOpts, clusterNodes) {
    return __awaiter(this, void 0, void 0, function* () {
        const info = yield execRedisCommand(redisOpts, (client) => client.info(), clusterNodes);
        return info;
    });
}
exports.getRedisInfo = getRedisInfo;
function getRedisClient(redisOpts, clusterNodes) {
    if (!redisClient) {
        if (clusterNodes && clusterNodes.length) {
            redisClient = new ioredis_1.Redis.Cluster(clusterNodes, redisOpts);
        }
        else {
            redisClient = new ioredis_1.Redis(redisOpts);
        }
        redisClient.on("error", (err) => {
            console.log(`${chalk.yellow("Redis:")} ${chalk.red("redis connection error")} ${err.message}`);
        });
        redisClient.on("connect", () => {
            console.log(`${chalk.yellow("Redis:")} ${chalk.green("connected to redis server")}`);
        });
        redisClient.on("end", () => {
            console.log(`${chalk.yellow("Redis:")} ${chalk.blueBright("disconnected from redis server")}`);
        });
    }
    return redisClient;
}
exports.getRedisClient = getRedisClient;
function execRedisCommand(redisOpts, cb, clusterNodes) {
    return __awaiter(this, void 0, void 0, function* () {
        const redisClient = getRedisClient(redisOpts, clusterNodes);
        const result = yield cb(redisClient);
        return result;
    });
}
exports.execRedisCommand = execRedisCommand;
function createQueue(foundQueue, redisOpts, opts = {}) {
    const { nodes, integrations } = opts;
    const createClient = function (type) {
        switch (type) {
            case "client":
                return getRedisClient(redisOpts, nodes);
            default:
                throw new Error(`Unexpected connection type: ${type}`);
        }
    };
    if (integrations && integrations[foundQueue.type]) {
        const integration = integrations[foundQueue.type];
        return {
            queue: integration.createQueue(foundQueue, redisOpts, nodes),
            responders: integration.responders,
        };
    }
    switch (foundQueue.type) {
        case "bullmq":
            return {
                queue: new bullmq_1.Queue(foundQueue.name, {
                    connection: getRedisClient(redisOpts, nodes),
                    prefix: foundQueue.prefix,
                }),
                responders: responders_1.BullMQResponders,
            };
        case "bull":
            return {
                queue: new Bull(foundQueue.name, {
                    createClient,
                    prefix: foundQueue.prefix,
                }),
                responders: responders_1.BullResponders,
            };
        default:
            console.error(chalk.red(`ERROR:`) +
                `Unexpected queue type: ${foundQueue.type} for queue ${foundQueue.name}`);
    }
}
exports.createQueue = createQueue;
